// k.yokomizo
module CTRL
  (  CLK,
     RESETB,
     START,
     ADJUST,
     LAP,
     Q_B,
     EN_A,
     CNT_EN,
     DATA_SEL,
     WREN,
     ADR,
     LAP_NO );

input CLK ;
input RESETB ;
input START ;
input ADJUST ;
input LAP ;
input [3:0]Q_B ;
input EN_A ;
output CNT_EN ;
output DATA_SEL ;
output WREN ;
output [4:0]ADR ;
output [4:0]LAP_NO ;

reg [1:0]STATE;
wire LAP_STOP;
reg [4:0]WR_ADR;
reg LAP_ADR;
reg [3:0] TMP_TIM;
reg DATA_SEL_SET;
reg HI_LEVEL;
reg  DATA_SEL;
reg [4:0]ADR ;

parameter [1:0]IDLE=2'b00;
parameter [1:0]COUNT=2'b01;
parameter [1:0]LAP_VIEW=2'b10;
//
always@(posedge CLK or negedge RESETB)
begin
   if(RESETB == 1'b0) //Reset
      STATE <= IDLE ;
   else               //CLK event
      case (STATE) 
        IDLE:
          if (START==1'b1)
            STATE <= COUNT;
          else
            if (LAP==1'b1) 
              STATE <= LAP_VIEW;
            else
              STATE <= IDLE;
        COUNT:
          if (START==1'b1)
            STATE <= IDLE;
          else
            STATE <= COUNT;   
        LAP_VIEW:
          if (START==1'b1) 
            STATE <= COUNT;
          else
            STATE <= LAP_VIEW;
        default:
          STATE <= IDLE;
      endcase  
end

assign CNT_EN = (STATE == COUNT)?1'b1:1'b0;
assign LAP_STOP = ((LAP==1'b1)||((START==1'b1)&&(STATE==COUNT)))?1'b1:1'b0;
  
always@(posedge CLK or negedge RESETB)
begin
   if(RESETB == 1'b0) // Reset
     begin
       WR_ADR  <= 5'b00000;
       LAP_ADR <= 5'b00000;
     end 
   else               // CLK event
      if (ADJUST==1'b1)
        begin 
          WR_ADR <= 5'b00000;
          LAP_ADR <=5'b00000; 
        end
      else
        if (LAP_STOP==1'b1) 
          if (STATE == COUNT)
            begin
              WR_ADR <= WR_ADR + 5'b00001;
              LAP_ADR <= LAP_ADR + 5'b00001;
            end
          else
            begin
              WR_ADR <= WR_ADR;
              if (LAP_ADR == WR_ADR)
                LAP_ADR <= 5'b00000 ;
              else
                LAP_ADR <= LAP_ADR + 5'b00001;
            end
end

assign WREN = ((LAP_STOP==1'b1)&&(STATE==COUNT))?1'b1:1'b0;

always@(posedge CLK or negedge RESETB)
begin
  if(RESETB == 1'b0 )  
     ADR <= 5'b00000;
   else            // CLK event
     if (STATE==COUNT)  
        ADR <= WR_ADR ;
     else 
        ADR <= LAP_ADR;
end

assign LAP_NO = LAP_ADR;

always@(posedge CLK or negedge RESETB)
begin
   if(RESETB == 1'b0)  // Reset
      begin
        DATA_SEL <= 1'b0;
        TMP_TIM  <= 4'b0000;
      end
   else                // CLK event
      if ((ADJUST==1'b1)||(START==1'b1))
        begin 
          DATA_SEL <= 1'b0;
          TMP_TIM <= 4'b0000;
        end
      else
        if (STATE == COUNT) 
          if ((DATA_SEL_SET==1'b1)&&(EN_A==1'b1))
            begin 
              DATA_SEL <= 1'b1;
              TMP_TIM <= Q_B;
            end
          else
            if ((TMP_TIM==Q_B)&&(EN_A==1'b1)) 
               DATA_SEL <=1'b0;
        else
          if (LAP==1'b1) 
            DATA_SEL <= 1'b1;
end

always@(posedge CLK or negedge RESETB)
begin
   if(RESETB == 1'b0)  // Reset
     DATA_SEL_SET <= 1'b0;
   else                // CLK event
     if ((ADJUST==1'b1)||(START==1'b1)) 
       DATA_SEL_SET <= 1'b0;
     else
       if (LAP==1'b1) 
         DATA_SEL_SET <= 1'b1;
       else
         if(EN_A==1'b1) 
           DATA_SEL_SET <= 1'b0;
         else
           DATA_SEL_SET <= DATA_SEL_SET ;
end

endmodule






