// k.yokomizo
module OUTPUT_IF
  (     CLK,
        RESETB,
        Q_A,
        Q_B,
        Q_C,
        Q_D,
        LAP_NO,
        WEN,
        CNT_EN,
        MEM_DATA,
        DATA_SEL,
        SEG_OUT,
        SEG_SEL
        );
parameter  [7:0]LED_CYCLE = 8'b11111111;        
input CLK;
input RESETB;
input [3:0]Q_A;
input [3:0]Q_B;
input [3:0]Q_C;
input [3:0]Q_D;
input [4:0]LAP_NO;
input WEN;
input CNT_EN;
input [15:0]MEM_DATA;
input DATA_SEL;
output [7:0]SEG_OUT;
output [5:0]SEG_SEL;

wire [3:0]LAP_NUM_A;
wire [3:0]LAP_NUM_B;
reg [3:0]SEG_DATA;
reg [3:0]SEG_DATA0;
reg [3:0]SEG_DATA1;
reg [3:0]SEG_DATA2;
reg [3:0]SEG_DATA3;
wire [3:0]SEG_DATA4;
wire [3:0]SEG_DATA5;
reg [7:0]LED_CNT_A;
reg [2:0]LED_CNT_B;
reg [15:0]LAP_DATA;
reg [7:0]SEG_OUT;
reg [5:0]SEG_SEL;

parameter HI_LEVEL = 1'b1;
//
//
always@(posedge CLK or negedge RESETB)
begin
  if(RESETB == 1'b0) 
     LAP_DATA <= 16'b0000000000000000;
   else // CLK event
     if (WEN==1'b1) 
        LAP_DATA <= {Q_D,Q_C,Q_B,Q_A};
end

always@(DATA_SEL or CNT_EN or MEM_DATA or LAP_DATA or Q_A or Q_B or Q_C or Q_D)
begin
   if (DATA_SEL==1'b0) 
     begin
       SEG_DATA0 <= Q_A; 
       SEG_DATA1 <= Q_B; 
       SEG_DATA2 <= Q_C; 
       SEG_DATA3 <= Q_D;
     end
   else
     if (CNT_EN==1'b1)
       begin
         SEG_DATA0 <= LAP_DATA[3:0]; 
         SEG_DATA1 <= LAP_DATA[7:4]; 
         SEG_DATA2 <= LAP_DATA[11:8]; 
         SEG_DATA3 <= LAP_DATA[15:12];
       end 
     else
       begin  
         SEG_DATA0 <= MEM_DATA[3:0]; 
         SEG_DATA1 <= MEM_DATA[7:4]; 
         SEG_DATA2 <= MEM_DATA[11:8]; 
         SEG_DATA3 <= MEM_DATA[15:12];
       end 
end

assign SEG_DATA4 = (DATA_SEL==1'b0)?4'b0000: LAP_NUM_A ;
assign SEG_DATA5 = (DATA_SEL==1'b0)?4'b0000: LAP_NUM_B ;

assign LAP_NUM_A = (LAP_NO[3:0] < 4'b1010)? LAP_NO[3:0]: LAP_NO[3:0]-4'b1010;
assign LAP_NUM_B = (LAP_NO[3:0] > 4'b1001)? 4'b0001:4'b0000;

always@(posedge CLK or negedge RESETB)
begin
  if(RESETB == 1'b0) // Reset
    begin
      LED_CNT_A <= 8'b00000000 ;
      LED_CNT_B <= 3'b000 ;
    end
  else               //CLK event
    if (LED_CNT_A==LED_CYCLE)
      begin     
        LED_CNT_A <= 8'b00000000;
        if (LED_CNT_B==3'b101) 
           LED_CNT_B <= 3'b000;
         else
           LED_CNT_B <= LED_CNT_B + 3'b001;
      end
    else
      begin
        LED_CNT_A <= LED_CNT_A + "00000001"; // count up
        LED_CNT_B <= LED_CNT_B; 
      end 
end

always@(posedge CLK or negedge RESETB)
begin
  if(RESETB == 1'b0)  // Reset
    begin
      SEG_DATA <= 4'b0000 ;
      SEG_SEL <=  6'b111111 ;
    end
  else                // CLK event
    case (LED_CNT_B) 
      3'b000:
        begin
          SEG_DATA <= SEG_DATA0;
          SEG_SEL <=6'b111110;
        end
      3'b001:
        begin
          SEG_DATA <= SEG_DATA1;
          SEG_SEL <=6'b111101;
        end
      3'b010:
        begin
          SEG_DATA <= SEG_DATA2;
          SEG_SEL <=6'b111011;
        end
      3'b011:
        begin
          SEG_DATA <= SEG_DATA3;
          SEG_SEL <=6'b110111;
        end
      3'b100:
        begin
          SEG_DATA <= SEG_DATA4;
          SEG_SEL <=6'b101111;
        end
      3'b101:
        begin
          SEG_DATA <= SEG_DATA5;
          SEG_SEL <=6'b011111;
        end
      default:
        begin
          SEG_DATA <= 4'b0000;
          SEG_SEL <=6'b111111;
        end
    endcase
end

always@(SEG_DATA)
begin
    case (SEG_DATA)
      4'b0000: SEG_OUT <= 8'b11111100;
      4'b0001: SEG_OUT <= 8'b01100000;
      4'b0010: SEG_OUT <= 8'b11011010;
      4'b0011: SEG_OUT <= 8'b11110010;
      4'b0100: SEG_OUT <= 8'b01100110;
      4'b0101: SEG_OUT <= 8'b10110110;
      4'b0110: SEG_OUT <= 8'b10111110;
      4'b0111: SEG_OUT <= 8'b11100100;
      4'b1000: SEG_OUT <= 8'b11111110; //8
      4'b1001: SEG_OUT <= 8'b11100110; //9
      4'b1010: SEG_OUT <= 8'b11101110; //A
      4'b1011: SEG_OUT <= 8'b00111110; //b
      4'b1100: SEG_OUT <= 8'b10011100; //c
      4'b1101: SEG_OUT <= 8'b01111010; //d
      4'b1110: SEG_OUT <= 8'b01011110; //e
      4'b1111: SEG_OUT <= 8'b10001110; //f
      default: SEG_OUT <= 8'b00000000;
    endcase
  end

endmodule







