// k.yokomizo
module SW_IF 
  (     CLK,
        RESETB,
        START_SW,
        ADJUST_SW,
        LAP_SW,
        START,
        ADJUST,
        LAP );

input  CLK;
input  RESETB;
input  START_SW;
input  ADJUST_SW;
input  LAP_SW;
output START;
output ADJUST;
output LAP;

reg [2:0]START_CHK;
reg START_DFF;
reg [2:0]ADJUST_CHK;
reg ADJUST_DFF;
reg [2:0]LAP_CHK;
reg LAP_DFF;
reg [3:0]SAMPLE_CNT;
wire SAMPLE_TIM;

always@(posedge CLK or negedge RESETB)
begin
   if(RESETB == 1'b0) 
     SAMPLE_CNT <= 4'b0000;
   else               // CLK event
     SAMPLE_CNT <= SAMPLE_CNT + "0001";
end

assign SAMPLE_TIM = (SAMPLE_CNT==4'b1111)?1'b1:1'b0;

always@(posedge CLK or negedge RESETB)
begin
   if(RESETB == 1'b0)  // Reset
     begin
       START_CHK  <= 3'b000 ;
       ADJUST_CHK <= 3'b000 ;
       LAP_CHK    <= 3'b000 ;
     end
   else                // CLK event
     if (SAMPLE_TIM==1'b1) 
       begin
         START_CHK  <= {START_CHK[1:0],START_SW};
         ADJUST_CHK <= {ADJUST_CHK[1:0],ADJUST_SW};
         LAP_CHK    <= {LAP_CHK[1:0], LAP_SW};
       end
end

always@(posedge CLK or negedge RESETB) 
begin
  if(RESETB == 1'b0)    // Reset
    begin
      START_DFF  <= 1'b0 ;
      ADJUST_DFF <= 1'b0 ;
      LAP_DFF    <= 1'b0 ;
    end
  else                 // CLK event
    begin
      if (START_CHK==3'b100) 
        START_DFF <= 1'b1;
      else
        START_DFF <= 1'b0;
      if (ADJUST_CHK==3'b100)         
        ADJUST_DFF <= 1'b1;
      else
        ADJUST_DFF <= 1'b0;
      if (LAP_CHK==3'b100) 
        LAP_DFF <= 1'b1;
      else
        LAP_DFF <= 1'b0;
    end
end

assign START  = ((START_CHK==3'b100)  && (START_DFF==1'b0))? 1'b1:1'b0; 
assign ADJUST = ((ADJUST_CHK==3'b100) && (ADJUST_DFF==1'b0))?1'b1:1'b0; 
assign LAP    = ((LAP_CHK==3'b100) && (LAP_DFF==1'b0))? 1'b1:1'b0; 

endmodule 





